#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from data_exports.compat import python_2_unicode_compatible


@python_2_unicode_compatible
class Format(models.Model):
    name = models.CharField(max_length=50)
    file_ext = models.CharField(max_length=10, blank=True)
    mime = models.CharField(max_length=50)
    template = models.CharField(max_length=255)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Export(models.Model):
    name = models.CharField(max_length=50)
    slug = models.SlugField(max_length=50, unique=True)
    model = models.ForeignKey(ContentType)
    export_format = models.ForeignKey(
        Format,
        blank=True,
        null=True,
        help_text=_(u"Leave empty to display as HTML"))

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('data_exports:export_view', kwargs={'slug': self.slug})

    def get_export_link(self):
        if self.slug:
            return '<a href="%s">Download</a>' % self.get_absolute_url()
        else:
            return _('Export not ready')
    get_export_link.allow_tags = True


@python_2_unicode_compatible
class Column(models.Model):
    export = models.ForeignKey(Export)
    column = models.CharField(max_length=255)
    label = models.CharField(max_length=255, blank=True)
    order = models.PositiveIntegerField()

    class Meta:
        ordering = ['order']

    def __str__(self):
        return self.column



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import template

from data_exports.compat import text_type


register = template.Library()


@register.filter()
def getattribute(model, item):
    """Chained lookup of item on model

    If item has dots (eg: 'foo.bar.baz'), recursively call getattribute():
    e = getattr(model, 'foo')
    e = getattr(e, 'bar')
    e = getattr(e, 'baz')
    At each step, check if e is a callable, and if so, use e()

    """
    elements = item.split('.')
    element = elements.pop(0)
    try:
        attr = getattr(model, element, None)
    except:
        return
    if attr is None:  # end of recursion
        return
    if callable(attr):
        try:
            attr = attr()
        except:  # couldn't call this method without params
            return
    if elements:
        return getattribute(attr, '.'.join(elements))
    return attr  # no elements left (end of recursion)


@register.filter()
def getvalue(dictionary, item):
    """Equivalent to dict.get(item), provided for convenience"""
    return dictionary.get(item)


@register.filter()
def nice_display(item):
    """Display a comma-separated list of models for M2M fields"""
    if hasattr(item, 'all'):  # RelatedManager: display a list
        return ', '.join(map(text_type, item.all()))
    return item



# -*- coding: utf-8 -*-

DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3'}}
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'data_exports',
]
SECRET_KEY = "not really secret, is it?"
ROOT_URLCONF = 'data_exports.test_urls'
MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)



DEBUG = True
TEMPLATE_DEBUG = True
DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3',
                         'NAME': 'test.sqlite'}}
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'data_exports',
    'south',
]
SECRET_KEY = "not really secret, is it?"
ROOT_URLCONF = 'data_exports.test_urls'



# -*- coding: utf-8 -*-

import sys

try:
    from django.utils.encoding import python_2_unicode_compatible
except ImportError:
    python_2_unicode_compatible = lambda x: x


text_type = str
if sys.version < '3':
    text_type = unicode  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from django.conf.urls.defaults import patterns, url
except ImportError:  # django 1.6 and above
    from django.conf.urls import patterns, url
from data_exports import views


urlpatterns = patterns(
    '',
    url(r'^add$', views.export_add, name='export_add'),
    url(r'^(?P<slug>[^/]+)$', views.export_view, name='export_view'),
    url(r'^(?P<slug>[^/]+)/columns$', views.export_cols, name='export_cols'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import IntegrityError, models
from django.forms.models import inlineformset_factory
from django.test import TestCase
from data_exports.templatetags import getter_tags as ttags
from data_exports.models import Format, Export, Column
from data_exports.forms import ColumnForm, ColumnFormSet, get_choices
from inspect_model import InspectModel

from data_exports.compat import text_type


class ExportTest(TestCase):

    def setUp(self):
        # create an export of the Export model (inception !)
        ct = ContentType.objects.get(app_label='data_exports', model='export')
        self.empty_export = Export.objects.create(name='test empty export',
                                                  slug='test-empty-export',
                                                  model=ct)
        self.im = InspectModel(Export)

        # create an export of the Export model with columns
        self.export = Export.objects.create(name='test export',
                                            slug='test-export',
                                            model=ct)
        for f in self.im.items:
            Column.objects.create(export=self.export,
                                  column=f,
                                  order=0)

        user = User.objects.create_user('admin', 'admin@admin.com', 'admin')
        user.is_superuser = True
        user.save()

    def test_slug(self):
        """Make sure the slug is unique."""
        with self.assertRaises(IntegrityError):
            Export.objects.create(name='foo',
                                  slug=self.empty_export.slug,
                                  model=self.empty_export.model)

    def test_column_choices(self):
        """Choices computed for the exported model

        When using the ColumnFormSet and the ColumnForm, all the accessible
        items (fields, relations, methods, attributes...) from the exported
        model are present in the "column" form field choices

        """
        class OneToOneToExport(models.Model):
            """Fake model.

            Make sure that get_choices works with
            SingleRelatedObjectDescriptor, as explained in ticket #4.

            """
            name = models.CharField(max_length=50)
            o2o = models.OneToOneField(Export)

        # reload the model's relations, to have the OneToOneToExport's relation
        # taken into account
        self.empty_export._meta._fill_related_objects_cache()
        self.empty_export._meta.init_name_map()

        ColumnInlineFormSet = inlineformset_factory(Export,
                                                    Column,
                                                    form=ColumnForm,
                                                    formset=ColumnFormSet)
        formset = ColumnInlineFormSet(instance=self.empty_export)
        # the column field has choices
        form = formset.forms[0]
        self.assertTrue(hasattr(form.fields['column'], 'choices'))
        choices = form.fields['column'].choices
        # all the model items are in the column field choices
        self.assertTrue(all([(i, i) in choices for i in self.im.items]))
        # and all the related model items are in the fields choices
        # export has a FK to Format named 'export_format'
        im_format = ['export_format.%s' % i
                     for i in InspectModel(Format).items]
        self.assertTrue(all([(i, i) in choices for i in im_format]))
        # export has a FK to ContentType named 'model'
        im_ct = ['model.%s' % i for i in InspectModel(ContentType).items]
        self.assertTrue(all([(i, i) in choices for i in im_ct]))
        # OneToOneToExport has a OneToOneField to ContentType named
        # 'onetoonetoexport'
        im_o2o = ['onetoonetoexport.%s' % i
                  for i in InspectModel(OneToOneToExport).items]
        self.assertTrue(all([(i, i) in choices for i in im_o2o]))
        # revert changes to name_map:  'unload' the OneToOneToExport relation
        del self.empty_export._meta._name_map['onetoonetoexport']

    def test_export_without_format(self):
        """Export without a format renders to a simple template"""
        self.client.login(username='admin', password='admin')
        # empty export
        resp = self.client.get(reverse('data_exports:export_view',
                               kwargs={'slug': self.empty_export.slug}))
        self.assertContains(resp, 'No columns where defined')
        # full export
        resp = self.client.get(reverse('data_exports:export_view',
                               kwargs={'slug': self.export.slug}))
        self.assertNotContains(resp, 'No columns where defined')
        for c in self.export.column_set.all():
            self.assertContains(resp, c.label if c.label else c)

    def test_export_with_format(self):
        """Export with a format gives a file download"""
        # create a format for a "naive csv export"
        csv_format = Format.objects.create(
            name='naive csv',
            file_ext='csv',
            mime='text/csv',
            template='data_exports/export_detail_csv.html')

        self.client.login(username='admin', password='admin')

        self.export.export_format = csv_format
        self.export.save()
        resp = self.client.get(reverse('data_exports:export_view',
                                       kwargs={'slug': self.export.slug}))
        self.assertEqual(resp['Content-Type'], self.export.export_format.mime)
        self.assertEqual(resp['Content-Disposition'],
                         'attachment; filename=%s.%s' % (
                             self.export.slug,
                             self.export.export_format.file_ext))

    def test_export_templatetag(self):
        """Templatetags provided for convenience"""
        # getattribute
        self.assertEqual(ttags.getattribute(self.export, 'model'),
                         getattr(self.export, 'model'))
        # getvalue
        d = {'foo': 'bar'}
        self.assertEqual(ttags.getvalue(d, 'foo'), d.get('foo'))
        # nice_display: displays a list for FK and ManyToMany
        column_list = ttags.nice_display(self.export.column_set).split(', ')
        for c in self.export.column_set.all():
            self.assertTrue(text_type(c) in column_list)
        # make sure getattribute and nice_display work on all choices
        for c, name in get_choices(Export):
            e = ttags.getattribute(self.export, c)
            ttags.nice_display(e)


class AdminTest(TestCase):

    def setUp(self):
        user = User.objects.create_user('admin', 'admin@admin.com', 'admin')
        user.is_staff = True
        user.is_superuser = True
        user.save()

    def test_create_export(self):
        self.client.login(username='admin', password='admin')
        ct = ContentType.objects.get(app_label='data_exports', model='export')
        resp = self.client.post(reverse('admin:data_exports_export_add'), {
            'name': 'test export',
            'slug': 'test-export',
            'model': ct.pk,
            '_save': 'Save'})
        # when creating, "save" is equivalent to "save and continue editing"
        self.assertRedirects(resp,
                             reverse('admin:data_exports_export_change',
                                     args=[1]))  # first export created, id=1

        # once an export is created, it's no more possible to modify its model
        resp = self.client.get(reverse('admin:data_exports_export_change',
                                       args=[1]))
        self.assertContains(resp, 'name="export_format"')
        self.assertNotContains(resp, 'name="model"')



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Deleting field 'Export.display_labels'
        db.delete_column('data_exports_export', 'display_labels')


    def backwards(self, orm):
        
        # Adding field 'Export.display_labels'
        db.add_column('data_exports_export', 'display_labels', self.gf('django.db.models.fields.BooleanField')(default=True), keep_default=False)


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'data_exports.column': {
            'Meta': {'ordering': "['order']", 'object_name': 'Column'},
            'column': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'export': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['data_exports.Export']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        'data_exports.export': {
            'Meta': {'object_name': 'Export'},
            'export_format': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['data_exports.Format']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'data_exports.format': {
            'Meta': {'ordering': "['name']", 'object_name': 'Format'},
            'attachment': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['data_exports']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import forms
from data_exports.models import Export, Column
from inspect_model import InspectModel


class ExportForm(forms.ModelForm):

    class Meta:
        model = Export
        exclude = ()

    def __init__(self, *args, **kwargs):
        super(ExportForm, self).__init__(*args, **kwargs)
        if self.instance:  # don't allow modification of the model once created
            del self.fields['model']


class ColumnForm(forms.ModelForm):
    # make sure we have a select widget for the column choice
    # Choice list is computed in data_exports.admin.ColumnFormSet
    column = forms.ChoiceField(choices=[])

    class Meta:
        model = Column
        exclude = ()


class ColumnFormSet(forms.models.BaseInlineFormSet):
    def get_choices(self):
        # avoid multiple choices generation for django 1.6+
        if not hasattr(self, "_choices"):
            model = self.instance.model.model_class()
            self._choices = [(u'', '---------')] + get_choices(model)
        return self._choices

    def add_fields(self, form, index):
        """Filter the form's column choices

        This is done at the formset level as there's no other way i could find
        to get the parent object (stored in self.instance), and the form at the
        same time.

        """
        super(ColumnFormSet, self).add_fields(form, index)
        form.fields['column'].choices = self.get_choices()


def get_choices(model, prefixes=[]):
    choices = []
    prefix = '.'.join(prefixes)
    if prefix:
        prefix = '%s.' % prefix
    im = InspectModel(model)
    items = ['%s%s' % (prefix, i) for i in im.items]
    choices += zip(items, items)
    for f in im.relation_fields:
        related_field = getattr(model, f)
        if hasattr(related_field, 'field'):  # ForeignKey
            related_model = related_field.field.rel.to
        else:
            related_model = related_field.related.model
        if f in prefixes:  # we already went through this model
            return []  # end of recursion
        new_prefixes = prefixes + [f]
        choices += get_choices(related_model, prefixes=new_prefixes)
    return choices



# -*- coding: utf-8 -*-
from distutils.core import setup
from os.path import abspath, dirname, join
from setuptools import find_packages


def read_relative_file(filename):
    """Returns contents of the given file, whose path is supposed relative
    to this module."""
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()


setup(
    name='django-data-exports',
    version=read_relative_file('VERSION').strip(),
    author=u'Mathieu agopian',
    author_email='mathieu.agopian@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/magopian/django-data-exports',
    license='BSD license, see LICENSE file',
    description='Model data exports for Django',
    long_description=open('README.rst').read(),
    install_requires=[
        'Django',
        'django-inspect-model',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import django
from django.contrib import admin
from data_exports.models import Export, Column, Format
from data_exports.forms import ColumnForm, ColumnFormSet


POST_URL_CONTINUE = None
if django.get_version() < '1.6':
    POST_URL_CONTINUE = '../%s/'


class ColumnInline(admin.TabularInline):
    extra = 0
    form = ColumnForm
    formset = ColumnFormSet
    model = Column


class ExportAdmin(admin.ModelAdmin):
    inlines = [ColumnInline]
    list_display = ['name', 'slug', 'model', 'export_format',
                    'get_export_link']
    list_filter = ['export_format', 'model']
    prepopulated_fields = {"slug": ("name",)}
    readonly_fields = ['model']
    search_fields = ['name', 'slug', 'model']

    def get_readonly_fields(self, request, obj=None):
        """The model can't be changed once the export is created"""
        if obj is None:
            return []
        return super(ExportAdmin, self).get_readonly_fields(request, obj)

    def get_formsets(self, request, obj=None):
        if obj is None:
            return
        if not hasattr(self, 'inline_instances'):
            self.inline_instances = self.get_inline_instances(request)
        for inline in self.inline_instances:
            yield inline.get_formset(request, obj)

    def response_add(self, request, obj, post_url_continue=POST_URL_CONTINUE):
        """If we're adding, save must be "save and continue editing"

        Two exceptions to that workflow:
        * The user has pressed the 'Save and add another' button
        * We are adding a user in a popup

        """
        if '_addanother' not in request.POST and '_popup' not in request.POST:
            request.POST['_continue'] = 1
        return super(ExportAdmin, self).response_add(request,
                                                     obj,
                                                     post_url_continue)

admin.site.register(Export, ExportAdmin)
admin.site.register(Format)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Column.label'
        db.alter_column(u'data_exports_column', 'label', self.gf('django.db.models.fields.CharField')(max_length=255))

    def backwards(self, orm):

        # Changing field 'Column.label'
        db.alter_column(u'data_exports_column', 'label', self.gf('django.db.models.fields.CharField')(max_length=50))

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'data_exports.column': {
            'Meta': {'ordering': "['order']", 'object_name': 'Column'},
            'column': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'export': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['data_exports.Export']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        u'data_exports.export': {
            'Meta': {'object_name': 'Export'},
            'export_format': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['data_exports.Format']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'})
        },
        u'data_exports.format': {
            'Meta': {'ordering': "['name']", 'object_name': 'Format'},
            'file_ext': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['data_exports']


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.forms.models import inlineformset_factory
from django.views.generic import DetailView
from django.views.generic import CreateView, UpdateView
from data_exports.forms import ExportForm, ColumnForm, ColumnFormSet
from data_exports.models import Export, Column


class ExportView(DetailView):
    model = Export

    def get_context_data(self, **kwargs):
        context = super(ExportView, self).get_context_data(**kwargs)
        model = self.object.model.model_class()
        qs = model.objects.all()
        if hasattr(model, "export_queryset"):
            qs = model.export_queryset(self.request)
        context['data'] = qs
        return context

    def render_to_response(self, context, **kwargs):
        resp = super(ExportView, self).render_to_response(context, **kwargs)
        export_format = self.object.export_format
        if export_format:
            filename = '%s.%s' % (self.object.slug, export_format.file_ext)
            resp['Content-Type'] = export_format.mime
            resp['Content-Disposition'] = 'attachment; filename=%s' % filename
        return resp

    def get_template_names(self, **kwargs):
        template_names = super(ExportView, self).get_template_names(**kwargs)
        export_format = self.object.export_format
        if export_format:
            template_names.insert(0, export_format.template)
        return template_names
export_view = login_required(ExportView.as_view())


class ExportAdd(CreateView):
    model = Export

    def get_success_url(self):
        return reverse('data_exports:export_cols',
                       kwargs={'slug': self.object.slug})
export_add = login_required(ExportAdd.as_view())


class ExportCols(UpdateView):
    column_inline = inlineformset_factory(Export,
                                          Column,
                                          form=ColumnForm,
                                          formset=ColumnFormSet)
    form_class = ExportForm
    model = Export
    template_name = 'data_exports/export_cols.html'

    def get_context_data(self, **kwargs):
        context = super(ExportCols, self).get_context_data(**kwargs)
        context['formset'] = self.column_inline(instance=self.object)
        if hasattr(self, 'formset'):  # formset was POSTed, but is invalid
            context['formset'] = self.formset
        return context

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            self.object = form.save()
            formset = self.column_inline(request.POST, instance=self.object)
            if formset.is_valid():
                formset.save()
                return self.form_valid(form)
            else:
                self.formset = formset
                return self.form_invalid(form)
        else:
            return self.form_invalid(form)
export_cols = login_required(ExportCols.as_view())



try:
    from django.conf.urls.defaults import patterns, url, include
except ImportError:  # django 1.6 and above
    from django.conf.urls import patterns, url, include
from django.contrib import admin


admin.autodiscover()


urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^exports/', include('data_exports.urls', namespace='data_exports')),
)
